@@ -24,7 +24,7 @@
  * @author Samuel Mimram
  */
 
-/* $Id: ssl_stubs.c 2951 2007-03-12 19:44:10Z smimram $ */
+/* $Id: ssl_stubs.c 5946 2008-10-01 11:09:28Z smimram $ */
 
 /*
  * WARNING: because of thread callbacks, all ssl functions should be in
@@ -61,9 +61,7 @@ static int client_verify_callback(int, X509_STORE_CTX *);
 static void finalize_ctx(value block)
 {
   SSL_CTX *ctx = Ctx_val(block);
-  caml_enter_blocking_section();
   SSL_CTX_free(ctx);
-  caml_leave_blocking_section();
 }
 
 static struct custom_operations ctx_ops =
@@ -83,9 +81,7 @@ static struct custom_operations ctx_ops =
 static void finalize_ssl_socket(value block)
 {
   SSL *ssl = SSL_val(block);
-  caml_enter_blocking_section();
   SSL_free(ssl);
-  caml_leave_blocking_section();
 }
 
 static struct custom_operations socket_ops =
@@ -509,9 +505,7 @@ CAMLprim value ocaml_ssl_get_cipher_version(value vcipher)
 static void finalize_cert(value block)
 {
   X509 *cert = Cert_val(block);
-  caml_enter_blocking_section();
   X509_free(cert);
-  caml_leave_blocking_section();
 }
 
 static struct custom_operations cert_ops =
@@ -563,7 +557,7 @@ CAMLprim value ocaml_ssl_get_certificate(value socket)
 
   CAMLlocal1(block);
   block = caml_alloc_final(2, finalize_cert, 0, 1);
-  Store_field(block, 1, (value)cert);
+  (*((X509 **) Data_custom_val(block))) = cert;
   CAMLreturn(block);
 }
 
